@@ -91,7 +91,6 @@ class_init (CamelFolderClass *camel_folder_class)
 
 	/* virtual method definition */
 	camel_folder_class->sync = folder_sync;
-	camel_folder_class->refresh_info = refresh_info;
 	camel_folder_class->expunge = exchange_expunge;
 	camel_folder_class->append_message = append_message;
 	camel_folder_class->get_message = get_message;
@@ -349,6 +348,7 @@ static GByteArray *
 get_message_data (CamelFolder *folder, const char *uid, CamelException *ex)
 {
 	CamelExchangeFolder *exch = CAMEL_EXCHANGE_FOLDER (folder);
+	CamelOfflineStore *offline = (CamelOfflineStore *) folder->parent_store;
 	CamelStream *stream, *stream_mem;
 	GByteArray *ba;
 
@@ -365,6 +365,12 @@ get_message_data (CamelFolder *folder, const char *uid, CamelException *ex)
 		return ba;
 	}
 
+	if (offline->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+                                     _("This message is not available in offline mode."));
+		return NULL;
+	}	
+
 	if (!camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_GET_MESSAGE,
 			      CAMEL_STUB_ARG_FOLDER, folder->full_name,
 			      CAMEL_STUB_ARG_STRING, uid,
@@ -399,6 +405,7 @@ get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 	ba = get_message_data (folder, uid, ex);
 	if (!ba)
 		return NULL;
+
 	stream = camel_stream_mem_new_with_byte_array (ba);
 
 	crlffilter = camel_mime_filter_crlf_new (CAMEL_MIME_FILTER_CRLF_DECODE, CAMEL_MIME_FILTER_CRLF_MODE_CRLF_ONLY);
@@ -611,7 +618,8 @@ transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 			if (!(message = get_message (source, camel_message_info_uid (info), ex)))
 				break;
 
-			camel_exchange_journal_append (journal, message, info, NULL, ex);
+			camel_exchange_journal_transfer (journal, (CamelExchangeFolder *)source, 
+							message, info, uids->pdata[i], NULL, ex);
 			camel_object_unref (message);
 
 			if (camel_exception_is_set (ex))
@@ -895,6 +903,7 @@ camel_exchange_folder_update_message_tag (CamelExchangeFolder *exch,
  * @name: the full name of the folder
  * @camel_flags: the folder flags passed to camel_store_get_folder().
  * @folder_dir: local directory this folder can cache data into
+ * @offline_state : offline status
  * @stub: the #CamelStub.
  * @ex: a #CamelException
  *
@@ -906,7 +915,7 @@ camel_exchange_folder_update_message_tag (CamelExchangeFolder *exch,
 gboolean
 camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 				 const char *name, guint32 camel_flags,
-				 const char *folder_dir,
+				 const char *folder_dir, int offline_state,
 				 CamelStub *stub, CamelException *ex)
 {
 	CamelExchangeFolder *exch = (CamelExchangeFolder *)folder;
@@ -991,7 +1000,7 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 			info = summary->pdata[i];
 			uids->pdata[i] = (char *)camel_message_info_uid (info);
 			flags->data[i] = ((CamelMessageInfoBase *)info)->flags & CAMEL_EXCHANGE_SERVER_FLAGS;
-			camel_tag_list_free (&((CamelMessageInfoBase *)info)->user_tags);
+			//camel_tag_list_free (&((CamelMessageInfoBase *)info)->user_tags);
 		}
 
 		camel_operation_start (NULL, _("Scanning for changed messages"));
@@ -1018,6 +1027,8 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 
 		camel_exchange_summary_set_readonly (folder->summary, folder_flags & CAMEL_STUB_FOLDER_READONLY);
 
+		if (offline_state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL )
+			return TRUE;
 		camel_operation_start (NULL, _("Fetching summary information for new messages"));
 		ok = camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_REFRESH_FOLDER,
 				      CAMEL_STUB_ARG_FOLDER, folder->full_name,
