@@ -58,7 +58,8 @@ static ExchangeAccountFolderResult create_folder (ExchangeHierarchy *hier,
 static ExchangeAccountFolderResult remove_folder (ExchangeHierarchy *hier,
 						  EFolder *folder);
 static ExchangeAccountFolderResult scan_subtree (ExchangeHierarchy *hier,
-						 EFolder *folder);
+						 EFolder *folder,
+						 gboolean offline);
 static void finalize (GObject *object);
 
 static void
@@ -335,9 +336,6 @@ create_folder (ExchangeHierarchy *hier, EFolder *parent,
 static ExchangeAccountFolderResult
 remove_folder (ExchangeHierarchy *hier, EFolder *folder)
 {
-	char *conf_key_cal="/apps/evolution/calendar/sources";
-	char *conf_key_tasks="/apps/evolution/tasks/sources";
-	char *conf_key_contacts="/apps/evolution/addressbook/sources";
 	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 	const char *folder_type, *physical_uri;
 
@@ -349,27 +347,27 @@ remove_folder (ExchangeHierarchy *hier, EFolder *folder)
         if (strcmp (folder_type, "calendar") == 0) {
                 cal_source_list = e_source_list_new_for_gconf (
                                         gconf_client_get_default (),
-                                        conf_key_cal);
-                remove_esource (hier->account, conf_key_cal, physical_uri,
-                                &cal_source_list, FALSE, FALSE);
+                                        CONF_KEY_CAL);
+                remove_esource (hier->account, EXCHANGE_CALENDAR_FOLDER, 
+				physical_uri, &cal_source_list, FALSE);
                 e_source_list_sync (cal_source_list, NULL);
                 g_object_unref (cal_source_list);
         }
         else if (strcmp (folder_type, "tasks") == 0) {
                 task_source_list = e_source_list_new_for_gconf (
                                         gconf_client_get_default (),
-                                        conf_key_tasks);
-                remove_esource (hier->account, conf_key_tasks, physical_uri,
-                                &task_source_list, FALSE, FALSE);
+                                        CONF_KEY_TASKS);
+                remove_esource (hier->account, EXCHANGE_TASKS_FOLDER, 
+				physical_uri, &task_source_list, FALSE);
                 e_source_list_sync (task_source_list, NULL);
                 g_object_unref (task_source_list);
         }
         else if (strcmp (folder_type, "contacts") == 0) {
                 cont_source_list = e_source_list_new_for_gconf (
                                         gconf_client_get_default (),
-                                        conf_key_contacts);
-                remove_esource (hier->account, conf_key_contacts, physical_uri,
-                                        &cont_source_list, FALSE, TRUE);
+                                        CONF_KEY_CONTACTS);
+                remove_esource (hier->account, EXCHANGE_CONTACTS_FOLDER, 
+				physical_uri, &cont_source_list, FALSE);
                 e_source_list_sync (cont_source_list, NULL);
                 g_object_unref (cont_source_list);
 	}
@@ -384,13 +382,13 @@ remove_folder (ExchangeHierarchy *hier, EFolder *folder)
 }
 
 static ExchangeAccountFolderResult
-scan_subtree (ExchangeHierarchy *hier, EFolder *folder)
+scan_subtree (ExchangeHierarchy *hier, EFolder *folder, gboolean offline)
 {
 	ExchangeAccountFolderResult folder_result;
 
 	check_hide_private (hier);
 
-	folder_result = EXCHANGE_HIERARCHY_CLASS (parent_class)->scan_subtree (hier, folder);
+	folder_result = EXCHANGE_HIERARCHY_CLASS (parent_class)->scan_subtree (hier, folder, offline);
 
 	if (exchange_hierarchy_is_empty (hier))
 		hierarchy_foreign_cleanup (hier);
@@ -435,9 +433,6 @@ exchange_hierarchy_foreign_add_folder (ExchangeHierarchy *hier,
 	const char *folder_type = NULL;
 	const char *physical_uri = NULL;
 	char *new_folder_name;
-	char *conf_key_cal="/apps/evolution/calendar/sources";
-	char *conf_key_tasks="/apps/evolution/tasks/sources";
-	char *conf_key_contacts="/apps/evolution/addressbook/sources";
 	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 
 	result =  create_internal (hier, hier->toplevel, folder_name, NULL, folder);
@@ -453,13 +448,12 @@ exchange_hierarchy_foreign_add_folder (ExchangeHierarchy *hier,
 		!(strcmp (folder_type, "calendar/public"))) {
 			cal_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						conf_key_cal);
+						CONF_KEY_CAL);
 			add_esource (hier->account,
-				     conf_key_cal,
+				     EXCHANGE_CALENDAR_FOLDER,
 				     new_folder_name,
 				     physical_uri,
-				     &cal_source_list,
-				     FALSE);
+				     &cal_source_list);
 			e_source_list_sync (cal_source_list, NULL);
 			g_object_unref (cal_source_list);
 
@@ -468,13 +462,12 @@ exchange_hierarchy_foreign_add_folder (ExchangeHierarchy *hier,
 			 !(strcmp (folder_type, "tasks/public"))) {
 				task_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						conf_key_tasks);
+						CONF_KEY_TASKS);
 				add_esource (hier->account,
-				     conf_key_tasks,
+				     EXCHANGE_TASKS_FOLDER,
 				     new_folder_name,
 				     physical_uri,
-				     &task_source_list,
-				     FALSE);
+				     &task_source_list);
 				e_source_list_sync (task_source_list, NULL);
 				g_object_unref (task_source_list);
 
@@ -484,14 +477,13 @@ exchange_hierarchy_foreign_add_folder (ExchangeHierarchy *hier,
 			 !(strcmp (folder_type, "contacts/ldap"))) {
 				cont_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						conf_key_contacts);
+						CONF_KEY_CONTACTS);
 
 				add_esource (hier->account,
-				     conf_key_contacts,
+				     EXCHANGE_CONTACTS_FOLDER,
 				     new_folder_name,
 				     physical_uri,
-				     &cont_source_list,
-				     TRUE);
+				     &cont_source_list);
 				e_source_list_sync (cont_source_list, NULL);
 				g_object_unref (cont_source_list);
 		}
